%  Brent R. Hickman and Timothy P. Hubbard
% hickmanbr@uchicago.edu and timothy.hubbard@colby.edu

% please cite our Journal of Applied Econometrics paper "Replacing Sample
% Trimming with Boundary Correction in Nonparametric Estimation of
% First-Price Auctions" if this code is helpful to you.

% compute mean squared error between kernel-smoothed density estimate and
% true density

function ise = computeise(v,vhat,a,b,gamma,bcind,kernel,b0,usehardle,N,T)

% true density
fbeta = betapdf(v,a,b);
f = gamma*1 + (1 - gamma)*fbeta;

% kernel-smoothed (estimated) density
if bcind == 1
	% left-boundary correction
	[fhat,hv_l] = kspdf_bc(vhat,kernel,v,b0,usehardle);
	% right-boundary correction
	[fhat_right,hv_r] = kspdf_bc(-vhat,kernel,-v,b0,usehardle);
	vind = find(v >= max(vhat) - hv_r);
	fhat(vind) = fhat_right(vind);
elseif bcind == 0
	[fhat,hf] = kspdf_no_bc(vhat,kernel,v,N*T,usehardle);
end

ise = (f - fhat).^2;

