%  Brent R. Hickman and Timothy P. Hubbard
% hickmanbr@uchicago.edu and timothy.hubbard@colby.edu

% please cite our Journal of Applied Econometrics paper "Replacing Sample
% Trimming with Boundary Correction in Nonparametric Estimation of
% First-Price Auctions" if this code is helpful to you.

% compute mean squared error between kernel-smoothed density estimate and
% true density

function ise = computeise_stg1(bid,bobs,bcind,kernel,b0,usehardle, ...
    bgrid,gbgrid)


NT = length(bobs); 

% need to evaluate the true bid density at arbitrary points that might be
% called within the integration routine.  to do this, send over a fine
% grid of bids with their true density values and then use interpolation.
f = interp1(bgrid,gbgrid,bid,'pchip');

% kernel-smoothed (estimated) density
if bcind == 1
	% left-boundary correction
	[fhat,hv_l] = kspdf_bc(bobs,kernel,bid,b0,usehardle);
	% right-boundary correction
	[fhat_right,hv_r] = kspdf_bc(-bobs,kernel,-bid,b0,usehardle);
	vind = find(bid >= max(bobs) - hv_r);
	fhat(vind) = fhat_right(vind);
elseif bcind == 0
	[fhat,hf] = kspdf_no_bc(bobs,kernel,bid,NT,usehardle);
end

ise = (f - fhat).^2;

