%  Brent R. Hickman and Timothy P. Hubbard
% hickmanbr@uchicago.edu and timothy.hubbard@colby.edu

% please cite our Journal of Applied Econometrics paper "Replacing Sample
% Trimming with Boundary Correction in Nonparametric Estimation of
% First-Price Auctions" if this code is helpful to you.

% compute mean squared error between kernel-smoothed density estimate and
% true density

function mise = computeise_stg1_postrun(bids,b_mat,bmin,bmax,bcind, ...
    kernel,b0,usehardle,bgrid,gbgrid)

% need to evaluate the true bid density at arbitrary points that might be
% called within the integration routine.  to do this, send over a fine
% grid of bids with their true density values and then use interpolation.
f = interp1(bgrid,gbgrid,bids,'pchip');


% identify which simulations span current bid
[S,NT] = size(b_mat);

for j=1:length(bids)
    ise = [];
    useind = [];
    buse = [];
    
    % take one bid in integral, find simulations that span bid and extract
    % only those
    bid = bids(j);
    useind = find((bmin <= bid) & (bmax >= bid));
    buse = b_mat(useind,:);
    npos = length(useind);
    
    if length(useind) > 0
        for i=1:length(useind)
            % observed bids from a given vector
            bobs = [];
            bobs = buse(i,:);
            
            % kernel-smoothed (estimated) density at current bid
            if bcind == 1
                % left-boundary correction
                [fhat,hv_l] = kspdf_bc(bobs,kernel,bid,b0,usehardle);
                % right-boundary correction
                [fhat_right,hv_r] = kspdf_bc(-bobs,kernel,-bid,b0,usehardle);
                % bring the two together to construct full estimate
                vind = find(bid >= max(bobs) - hv_r);
                fhat(vind) = fhat_right(vind);
            elseif bcind == 0
                [fhat,hf] = kspdf_no_bc(bobs,kernel,bid,NT,usehardle);
            end
            % compute ise for simulation i
            ise(i) = (f(j) - fhat).^2;
        end
    else
        % if no simulations span valuation---fhat = 0 for all simulations
        ise = f(j).^2;
	end
    % mise including zeros
    nzero = S - npos;
    wpos = npos/S;
    wzero = nzero/S;
    mise(j) = wpos*mean(ise) + wzero*f(j).^2;
end