%  Brent R. Hickman and Timothy P. Hubbard
% hickmanbr@uchicago.edu and timothy.hubbard@colby.edu

% please cite our Journal of Applied Econometrics paper "Replacing Sample
% Trimming with Boundary Correction in Nonparametric Estimation of
% First-Price Auctions" if this code is helpful to you.

% compute mean squared error between kernel-smoothed density estimate and
% true density

function mise = computeise_stg2_postrun(vals,vhat_mat,vmin,vmax, ...
    a,b,gamma,bcind,kernel,b0,usehardle,N,T)

% true density
fbeta = betapdf(vals,a,b);
f = gamma*1 + (1 - gamma)*fbeta;

S = size(vhat_mat,1);

for j=1:length(vals)
    ise = [];
    useind = [];
    vuse = [];

    % take one value in integral, find simulations that span value and
    % extract only those
    v = vals(j);
    useind = find((vmin <= v) & (vmax >= v));
    vuse = vhat_mat(useind,:);
    npos = length(useind);
    
    if npos > 0
        for i=1:length(useind)
            % pseudo values from a given vector
            vhat = [];
            vhat = vuse(i,:);
            
            % kernel-smoothed (estimated) density
            if bcind == 1
                % left-boundary correction
                [fhat,hv_l] = kspdf_bc(vhat,kernel,v,b0,usehardle);
                % right-boundary correction
                [fhat_right,hv_r] = kspdf_bc(-vhat,kernel,-v,b0,usehardle);
                % bring the two together to construct full estimate
                vind = find(v >= max(vhat) - hv_r);
                fhat(vind) = fhat_right(vind);
            elseif bcind == 0
                [fhat,hf] = kspdf_no_bc(vhat,kernel,v,N*T,usehardle);
            end
            % compute ise for simulation i
            ise(i) = (f(j) - fhat).^2;
        end
    else
        % if no simulations span valuation---fhat = 0 for all simulations
        ise = f(j).^2;
	end
    % mise including zeros
    nzero = S - npos;
    wpos = npos/S;
    wzero = nzero/S;
    mise(j) = wpos*mean(ise) + wzero*f(j).^2;
end