%  Brent R. Hickman and Timothy P. Hubbard
% hickmanbr@uchicago.edu and timothy.hubbard@colby.edu

% please cite our Journal of Applied Econometrics paper "Replacing Sample
% Trimming with Boundary Correction in Nonparametric Estimation of
% First-Price Auctions" if this code is helpful to you.

% process Monte Carlo output mat files

clear all
clc

% parameters for figure properties
lwidth = 2;
fsize = 16;
set(0,'defaulttextinterpreter','latex','Defaulttextfontsize',fsize);

% switch to print some output to screen or just in latex table format
printtoscreen = 1;
% switch to plot stage 1 stuff
plotstg1 = 0;

firststageSH = 0
firststageHH = 1

evalgrid = [];
ftruegrid = [];
vgridpts = [];

bgridpts = [];

% define folder path to draw results from
folder = '';
outfolder = '';

for iii=1:4
	fname = strcat(folder,'beta_mc_ex',num2str(iii),'.mat');
	load(fname);
	saverun = 0
	
	vlowbiasHH = mean(vmin_trimHH) - vlow;
	vhighbiasHH = mean(vmax_trimHH) - vhigh;
	vlowbiasSH = mean(vmin_trimSH) - vlow;
	vhighbiasSH = mean(vmax_trimSH) - vhigh;
	vlowbiasbc = mean(vmin_bc) - vlow;
	vhighbiasbc = mean(vmax_bc) - vhigh;
	
	if plotstg1 == 1
		% ploting figures like GPV has for their MC study
		figheight = max([max(g95_trimHH) max(g95_trimSH) max(g95_bc)])*1.1;
		figure
		set(gcf,'DefaultLineLineWidth',lwidth)
		set(gca,'FontSize',fsize)
		plot(b_evalpts,gbtrue,b_evalpts,gmed_trimHH,'--', ...
			b_evalpts,g5_trimHH,'--',b_evalpts,g95_trimHH,'--')
		hold all
		axis([vlow bhigh 0 figheight])
		ymax = max(ylim);
		bbar_low = mean(bmin + hg_trimHH);
		bbar_high = mean(bmax - hg_trimHH);
		plot([bbar_low bbar_low],[0 ymax],'k',[bbar_high bbar_high],[0 ymax],'k')
		xlabel('$\mathbf{B}$','FontSize',fsize)
		ylabel('$\mathbf{\hat{g}_B(b)}$','FontSize',fsize)
		if saverun == 1
			fname = strcat(outfolder,'beta_mc_stg1_trimHH_ex',num2str(z));
			fnameeps = strcat(fname,'.eps');
			print('-depsc',fnameeps)
		end
		
		figure
		set(gcf,'DefaultLineLineWidth',lwidth)
		set(gca,'FontSize',fsize)
		plot(b_evalpts,gbtrue,b_evalpts,gmed_trimSH,'--', ...
			b_evalpts,g5_trimSH,'--',b_evalpts,g95_trimSH,'--')
		hold all
		axis([vlow bhigh 0 figheight])
		ymax = max(ylim);
		bbar_low = mean(bmin + hg_trimSH);
		bbar_high = mean(bmax - hg_trimSH);
		plot([bbar_low bbar_low],[0 ymax],'k',[bbar_high bbar_high],[0 ymax],'k')
		xlabel('$\mathbf{B}$','FontSize',fsize)
		ylabel('$\mathbf{\hat{g}_B(b)}$','FontSize',fsize)
		if saverun == 1
			fname = strcat(outfolder,'beta_mc_stg1_trimSH_ex',num2str(z));
			fnameeps = strcat(fname,'.eps');
			print('-depsc',fnameeps)
		end
		
		figure
		set(gcf,'DefaultLineLineWidth',lwidth)
		set(gca,'FontSize',fsize)
		plot(b_evalpts,gbtrue,b_evalpts,gmed_bc,'--',b_evalpts,g5_bc,'--', ...
			b_evalpts,g95_bc,'--')
		hold all
		axis([vlow bhigh 0 figheight])
		ymax = max(ylim);
		bbar_low = mean(bmin);
		bbar_high = mean(bmax);
		plot([bbar_low bbar_low],[0 ymax],'k',[bbar_high bbar_high],[0 ymax],'k')
		xlabel('$\mathbf{B}$','FontSize',fsize)
		ylabel('$\mathbf{\hat{g}_B(b)}$','FontSize',fsize)
		if saverun == 1
			fname = strcat(outfolder,'beta_mc_stg1_bc_ex',num2str(z));
			fnameeps = strcat(fname,'.eps');
			print('-depsc',fnameeps)
		end
    end
    	
	for i=1:neval
		ind = [];
		if i >= neval/2
			ind = find(vmax_bc >= evalpts(i));
		elseif i < neval/2
			ind = find(vmin_bc <= evalpts(i));
		end
		fmean_bc(i) = mean(fV_bc_matrix(ind,i));
		fmed_bc(i) = median(fV_bc_matrix(ind,i));
		f5_bc(i) =  prctile(fV_bc_matrix(ind,i),5);
		f95_bc(i) =  prctile(fV_bc_matrix(ind,i),95);
		ind = [];
		if i >= neval/2
			% the _no_bc values involve no trimming---they are the highest
			% and lowest implied pseudo values given bids without trimming;
			% the _trim values values are the highest and lowest pseudo
			% values from a given sample which are in the GPV region---the
			% highest and lowest implied pseudo values corresponding to a
			% sample of bids that are not within a bandwidth of the highest
			% and lowest pseudo values.  in contrast lowline (hiline) is
			% the lowest (highest) value defining the GPV region and it is
			% likely not observed in the data.
			ind = find(vmax_no_bcHH >= evalpts(i));
		elseif i < neval/2
			ind = find(vmin_no_bcHH <= evalpts(i));
		end
		fmean_trimHH(i) = mean(fV_trimHH_matrix(ind,i));
		fmed_trimHH(i) = median(fV_trimHH_matrix(ind,i));
		f5_trimHH(i) =  prctile(fV_trimHH_matrix(ind,i),5);
		f95_trimHH(i) =  prctile(fV_trimHH_matrix(ind,i),95);
		ind = [];
		if i >= neval/2
			ind = find(vmax_no_bcSH >= evalpts(i));
		elseif i < neval/2
			ind = find(vmin_no_bcSH <= evalpts(i));
		end
		fmean_trimSH(i) = mean(fV_trimSH_matrix(ind,i));
		fmed_trimSH(i) = median(fV_trimSH_matrix(ind,i));
		f5_trimSH(i) =  prctile(fV_trimSH_matrix(ind,i),5);
		f95_trimSH(i) =  prctile(fV_trimSH_matrix(ind,i),95);
	end
	
		
	figheight = max([max(f95_bc) max(f95_trimHH) max(f95_trimSH)])*1.1;
	fmed_trimHH(isnan(fmed_trimHH) == 1) = 0;
	f5_trimHH(isnan(f5_trimHH) == 1) = 0;
	f95_trimHH(isnan(f95_trimHH) == 1) = 0;
	% plot some GPV-like figures
	figure
	set(gcf,'DefaultLineLineWidth',lwidth)
	set(gca,'FontSize',fsize)
	plot(evalpts,ftrue,evalpts,fmed_trimHH,'--',evalpts,f5_trimHH,'--', ...
		evalpts,f95_trimHH,'--')
	vbar_low = mean(lowlineHH + hfHH);
	vbar_high = mean(hilineHH - hfHH);
	hold all
	axis([vlow vhigh 0 figheight])
	ymax = max(ylim);
	plot([vbar_low vbar_low],[0 ymax],'k',[vbar_high vbar_high],[0 ymax],'k')
	xlabel('$\mathbf{V}$','FontSize',fsize)
	ylabel('DENSITY','FontSize',fsize)
	if saverun == 1
		fname = strcat(outfolder,'beta_mc_trimHH_ex',num2str(z));
		fnameeps = strcat(fname,'.eps');
		print('-depsc',fnameeps)
	end
    coverHHhigh = gamma*vbar_high + (1 - gamma)*betacdf(vbar_high,az,bz);
    coverHHlow = gamma*vbar_low + (1 - gamma)*betacdf(vbar_low,az,bz);
    coverHH = coverHHhigh - coverHHlow;
	
	fmed_trimSH(isnan(fmed_trimSH) == 1) = 0;
	f5_trimSH(isnan(f5_trimSH) == 1) = 0;
	f95_trimSH(isnan(f95_trimSH) == 1) = 0;
	figure
	set(gcf,'DefaultLineLineWidth',lwidth)
	set(gca,'FontSize',fsize)
	plot(evalpts,ftrue,evalpts,fmed_trimSH,'--',evalpts,f5_trimSH,'--', ...
		evalpts,f95_trimSH,'--')
	vbar_low = mean(lowlineSH + hfSH);
	vbar_high = mean(hilineSH - hfSH);
	hold all
	axis([vlow vhigh 0 figheight])
	ymax = max(ylim);
	plot([vbar_low vbar_low],[0 ymax],'k',[vbar_high vbar_high],[0 ymax],'k')
	xlabel('$\mathbf{V}$','FontSize',fsize)
	ylabel('DENSITY','FontSize',fsize)
	if saverun == 1
		fname = strcat(outfolder,'beta_mc_trimSH_ex',num2str(z));
		fnameeps = strcat(fname,'.eps');
		print('-depsc',fnameeps)
	end
    coverSHhigh = gamma*vbar_high + (1 - gamma)*betacdf(vbar_high,az,bz);
    coverSHlow = gamma*vbar_low + (1 - gamma)*betacdf(vbar_low,az,bz);
    coverSH = coverSHhigh - coverSHlow;
	
	fmed_bc(isnan(fmed_bc) == 1) = 0;
	f5_bc(isnan(f5_bc) == 1) = 0;
	f95_bc(isnan(f95_bc) == 1) = 0;
	figure
	set(gcf,'DefaultLineLineWidth',lwidth)
	set(gca,'FontSize',fsize)
	plot(evalpts,ftrue,evalpts,fmed_bc,'--',evalpts,f5_bc,'--', ...
		evalpts,f95_bc,'--')
	hold all
	axis([vlow vhigh 0 figheight])
	ymax = max(ylim);
	vbar_low = mean(vmin_bc);
	vbar_high = mean(vmax_bc);
	plot([vbar_low vbar_low],[0 ymax],'k',[vbar_high vbar_high],[0 ymax],'k')
	xlabel('$\mathbf{V}$','FontSize',fsize)
	ylabel('DENSITY','FontSize',fsize)
	if saverun == 1
		fname = strcat(outfolder,'beta_mc_bc_ex',num2str(z));
		fnameeps = strcat(fname,'.eps');
		print('-depsc',fnameeps)
	end
    coverbchigh = gamma*vbar_high + (1 - gamma)*betacdf(vbar_high,az,bz);
    coverbclow = gamma*vbar_low + (1 - gamma)*betacdf(vbar_low,az,bz);
    coverbc = coverbchigh - coverbclow;
	
	% edf of vlow estimator---uses vmin_trim values which represent the
	% lowest pseudo value corresponding to an observed bid that survives
	% trimming
	vminHH = sort(vmin_trimHH);
	vminSH = sort(vmin_trimSH);
	vminBC = sort(vmin_bc);
	vlowedfHH = kscdf(vminHH,'edf',vminHH);
	vlowedfSH = kscdf(vminSH,'edf',vminSH);
	vlowedfBC = kscdf(vminBC,'edf',vminBC);
	figure
	set(gcf,'DefaultLineLineWidth',lwidth)
	set(gca,'FontSize',fsize)
	plot(vminBC,vlowedfBC,vminSH,vlowedfSH,'--', ...
		vminHH,vlowedfHH,'-.')
	xlabel('$\hat{\underline{V}}$','FontSize',fsize)
	ylabel('CDF OF $\hat{\underline{V}}$','FontSize',fsize)
	legend('BC','TRIM $\mathbf{(h^{b*},h^{v*})}$\quad', ...
        'TRIM $\mathbf{(h^{bG},h^{v*})}$\quad', ...
		'FontSize',fsize,'Location','NorthOutside','Orientation','horizontal')
	if saverun == 1
		fname = strcat(outfolder,'vlowedf_ex',num2str(z));
		fnameeps = strcat(fname,'.eps');
		print('-depsc',fnameeps)
	end	
	
	% edf of vhigh estimator---uses vmax_trim values which represent the
	% highest pseudo value corresponding to an observed bid that survives
	% trimming
	vmaxHH = sort(vmax_trimHH);
	vmaxSH = sort(vmax_trimSH);
	vmaxBC = sort(vmax_bc);
	vhighedfHH = kscdf(vmaxHH,'edf',vmaxHH);
	vhighedfSH = kscdf(vmaxSH,'edf',vmaxSH);
	vhighedfBC = kscdf(vmaxBC,'edf',vmaxBC);
	figure
	set(gcf,'DefaultLineLineWidth',lwidth)
	set(gca,'FontSize',fsize)
	plot(vmaxBC,vhighedfBC,vmaxSH,vhighedfSH,'--', ...
		vmaxHH,vhighedfHH,'-.')
	xlabel('$\hat{\overline{V}}$','FontSize',fsize)
	ylabel('CDF OF $\hat{\overline{V}}$','FontSize',fsize)
	legend('BC','TRIM $\mathbf{(h^{b*},h^{v*})}$\quad', ...
        'TRIM $\mathbf{(h^{bG},h^{v*})}$\quad', ...
		'FontSize',fsize,'Location','NorthOutside','Orientation','horizontal')
	if saverun == 1
		fname = strcat(outfolder,'vhighedf_ex',num2str(z));
		fnameeps = strcat(fname,'.eps');
		print('-depsc',fnameeps)
	end

    evalgrid = [evalgrid; evalpts];
    ftruegrid = [ftruegrid; ftrue];
    
    vgridpts = [vgridpts; vgrid];
    bgridpts = [bgridpts; bgrid];
	
	
	fprintf('\n\nOUTPUT for EX %d\n',z)
	if printtoscreen == 1
		fprintf('\nHH trimming throws out %.0f bids on average\n',mean(ntrimHH))
		fprintf('HH trimming mean MSEP: %.4f\n',mean(msep_trimHH))
		fprintf('HH trimming bias of vlow: %.4f\n',vlowbiasHH)
		fprintf('HH trimming bias of vbar: %.4f\n',vhighbiasHH)
		fprintf('HH trimming RMISE of g_B: %.4f\n',sqrt(stg1_mise_trimHH))
		fprintf('HH trimming RMISE of f_V in GPV HH region: %.4f\n',sqrt(gpvregion_mise_HH_HHreg))
		fprintf('HH trimming RMISE of f_V in GPV SH region: %.4f\n',sqrt(gpvregion_mise_HH_SHreg))
		fprintf('HH trimming RMISE of f_V: %.4f\n',sqrt(mise_trimHH))
		
		fprintf('\nSH trimming throws out %.0f bids on average\n',mean(ntrimSH))
		fprintf('SH trimming mean MSEP: %.4f\n',mean(msep_trimSH))
		fprintf('SH trimming bias of vlow: %.4f\n',vlowbiasSH)
		fprintf('SH trimming bias of vbar: %.4f\n',vhighbiasSH)
		fprintf('SH trimming RMISE of g_B: %.4f\n',sqrt(stg1_mise_trimSH))
		fprintf('SH trimming RMISE of f_V in GPV HH region: %.4f\n',sqrt(gpvregion_mise_SH_HHreg))
		fprintf('SH trimming RMISE of f_V in GPV SH region: %.4f\n',sqrt(gpvregion_mise_SH_SHreg))
		fprintf('SH trimming RMISE of f_V: %.4f\n',sqrt(mise_trimSH))
		
		fprintf('\nBC mean MSEP: %.4f\n',mean(msep_bc))
		fprintf('BC bias of vlow: %.4f\n',vlowbiasbc)
		fprintf('BC bias of vbar: %.4f\n',vhighbiasbc)
		fprintf('BC RMISE of g_B: %.4f\n',sqrt(stg1_mise_bc))
		fprintf('BC RMISE of f_V in GPV HH region: %.4f\n',sqrt(gpvregion_mise_bc_HHreg))
		fprintf('BC RMISE of f_V in GPV SH region: %.4f\n',sqrt(gpvregion_mise_bc_SHreg))
		fprintf('BC RMISE of f_V: %.4f\n',sqrt(mise_bc))
	end
    
    if firststageHH == 1
        % some first-stage mise calculations
        minbmin = vlow + mean(hg_trimHH);
        maxbmax = bhigh - mean(hg_trimHH);
        % compute stage 1 mise
        stg1_mise_bc = quad('computeise_stg1_postrun',minbmin,maxbmax, ...
            quadtol,[],b_mat,bmin,bmax,usebc,kernel,b0,usehardle, ...
            bgrid,gbgrid);
        stg1_mise_trimHH = quad('computeise_stg1_postrun',minbmin,maxbmax, ...
            quadtol,[],b_mat,bmin,bmax,usetrim,kernel,b0,usehardle, ...
            bgrid,gbgrid);
        stg1_mise_trimSH = quad('computeise_stg1_postrun',minbmin,maxbmax, ...
            quadtol,[],b_mat,bmin,bmax,usetrim,kernel,b0,usesilver, ...
            bgrid,gbgrid);
        fprintf('\nFirst-stage RMISE use [blow+h^{b*},bhigh-h^{b*}], EXAMPLE %d:\n',z)
        fprintf('HH trimming: \t%.4f\n',sqrt(stg1_mise_trimHH))
        fprintf('SH trimming: \t%.4f\n',sqrt(stg1_mise_trimSH))
        fprintf('BC: \t\t%.4f\n',sqrt(stg1_mise_bc))
    end
    if firststageSH == 1
        minbmin = vlow + mean(hg_trimSH);
        maxbmax = bhigh - mean(hg_trimSH);
        % compute stage 1 mise
        stg1_mise_bc = quad('computeise_stg1_postrun',minbmin,maxbmax, ...
            quadtol,[],b_mat,bmin,bmax,usebc,kernel,b0,usehardle, ...
            bgrid,gbgrid);
        stg1_mise_trimHH = quad('computeise_stg1_postrun',minbmin,maxbmax, ...
            quadtol,[],b_mat,bmin,bmax,usetrim,kernel,b0,usehardle, ...
            bgrid,gbgrid);
        stg1_mise_trimSH = quad('computeise_stg1_postrun',minbmin,maxbmax, ...
            quadtol,[],b_mat,bmin,bmax,usetrim,kernel,b0,usesilver, ...
            bgrid,gbgrid);
        fprintf('\nFirst-stage RMISE use [blow+h^{bG},bhigh-h^{bG}], EXAMPLE %d:\n',z)
        fprintf('HH trimming: \t%.4f\n',sqrt(stg1_mise_trimHH))
        fprintf('SH trimming: \t%.4f\n',sqrt(stg1_mise_trimSH))
        fprintf('BC: \t\t%.4f\n',sqrt(stg1_mise_bc))
	end

    fprintf('\nCoverage of mass in density results (on average), EXAMPLE %d:\n',z)
    fprintf('HH trimming: \t%.2f\n',coverHH*100)
    fprintf('SH trimming: \t%.2f\n',coverSH*100)
    fprintf('BC: \t\t%.2f\n',coverbc*100)
end



% pdfs used in MC study
figure
box on
set(gcf,'DefaultLineLineWidth',lwidth)
set(gca,'FontSize',fsize)
hold all
hi = 20;
ind = 1:hi:length(evalgrid);
ind = [ind length(evalgrid)];
plot(evalgrid(1,ind),ftruegrid(1,ind),'--')
plot(evalgrid(2,ind),ftruegrid(2,ind),'-s')
plot(evalgrid(3,ind),ftruegrid(3,ind),'-x')
plot(evalgrid(4,ind),ftruegrid(4,ind),'-o')
xlabel('$\mathbf{V}$','FontSize',fsize)
ylabel('$\mathbf{f_i(v)}$','FontSize',fsize)
xlim([vlow vhigh])
legend('$\mathbf{f_1(v)}$','$\mathbf{f_2(v)}$', ...
    '$\mathbf{f_3(v)}$','$\mathbf{f_4(v)}$', ...
		'FontSize',fsize,'Location','Northeast')
if saverun == 1
    fnameeps = strcat(outfolder,'mc_dists.eps');
	print('-depsc',fnameeps)
end

figure
box on
set(gcf,'DefaultLineLineWidth',lwidth)
set(gca,'FontSize',fsize)
hold all
hi = 4000;
ind = 1:hi:length(vgridpts);
ind = [ind length(vgridpts)];
plot(vgridpts(1,ind),bgridpts(1,ind),'--')
plot(vgridpts(2,ind),bgridpts(2,ind),'-s')
plot(vgridpts(3,ind),bgridpts(3,ind),'-x')
plot(vgridpts(4,ind),bgridpts(4,ind),'-o')
plot(vgridpts(4,:),vgridpts(4,:),'k')
xlabel('$\mathbf{V}$','FontSize',fsize)
ylabel('$\mathbf{B}$','FontSize',fsize)
axis tight
legend('$\mathbf{\beta_1(v)}$','$\mathbf{\beta_2(v)}$', ...
    '$\mathbf{\beta_3(v)}$','$\mathbf{\beta_4(v)}$', ...
		'FontSize',fsize,'Location','Northwest')
if saverun == 1
	fnameeps = strcat(outfolder,'mc_bfcns.eps');
	print('-depsc',fnameeps)
end
